use gpapi::auth::SamlAuthData;
use log::error;
use tokio::sync::{mpsc, RwLock};

pub enum AuthError {
  /// Failed to load page due to TLS error
  TlsError,
  /// 1. Found auth data in headers/body but it's invalid
  /// 2. Loaded an empty page, failed to load page. etc.
  Invalid,
  /// No auth data found in headers/body
  NotFound,
  /// Other errors
  Other,
}

pub type AuthResult = anyhow::Result<SamlAuthData, AuthError>;

pub struct AuthMessenger {
  tx: mpsc::UnboundedSender<AuthResult>,
  rx: RwLock<mpsc::UnboundedReceiver<AuthResult>>,
}

impl AuthMessenger {
  pub fn new() -> Self {
    let (tx, rx) = mpsc::unbounded_channel();

    Self {
      tx,
      rx: RwLock::new(rx),
    }
  }

  pub async fn recv_auth_data(&self) -> AuthResult {
    let mut rx = self.rx.write().await;
    rx.recv().await.unwrap_or_else(|| Err(AuthError::Other))
  }

  pub fn send_auth_result(&self, result: AuthResult) {
    if let Err(err) = self.tx.send(result) {
      error!("Failed to send auth result: {}", err);
    }
  }

  pub fn send_auth_error(&self, err: AuthError) {
    self.send_auth_result(Err(err));
  }

  pub fn send_auth_data(&self, data: SamlAuthData) {
    self.send_auth_result(Ok(data));
  }
}
